@@ -1,10 +1,9 @@
 /*  GNU ddrescue - Data recovery tool
-    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012,
-    2013, 2014 Antonio Diaz Diaz.
+    Copyright (C) 2004-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -19,6 +18,7 @@
 #define _FILE_OFFSET_BITS 64
 
 #include <algorithm>
+#include <cerrno>
 #include <climits>
 #include <cstdio>
 #include <cstring>
@@ -26,19 +26,51 @@
 #include <string>
 #include <vector>
 #include <stdint.h>
+#include <unistd.h>
 
 #include "block.h"
-#include "ddrescue.h"
+#include "mapbook.h"
 
 
-// Return values: 1 write error, 0 OK, -1 interrupted, -2 logfile error.
+// Return values: 1 write error, 0 OK.
 //
-int Fillbook::fill_areas( const std::string & filltypes )
+int Fillbook::fill_block( const Sblock & sb )
+  {
+  if( sb.size() <= 0 || sb.size() > softbs() )
+    internal_error( "bad size filling a Block." );
+  const int size = sb.size();
+
+  if( write_location_data )	// write location data into each sector
+    for( long long pos = sb.pos(); pos < sb.end(); pos += hardbs() )
+      {
+      char * const buf = (char *)iobuf() + ( pos - sb.pos() );
+      const int bufsize = std::min( 80LL, sb.end() - pos );
+      const int len = snprintf( buf, bufsize,
+                                "\n# position      sector  status\n"
+                                "0x%08llX  0x%08llX  %c\n",
+                                pos, pos / hardbs(), sb.status() );
+      if( len > 0 && len < bufsize )
+        std::memset( buf + len, ' ', bufsize - len );
+      }
+  if( writeblock( odes_, iobuf(), size, sb.pos() + offset() ) != size ||
+      ( synchronous_ && fsync( odes_ ) < 0 && errno != EINVAL ) )
+    {
+    if( !ignore_write_errors ) final_msg( "Write error", errno );
+    return 1;
+    }
+  filled_size += size; remaining_size -= size;
+  return 0;
+  }
+
+
+// Return values: 1 write error, 0 OK, -1 interrupted, -2 mapfile error.
+//
+int Fillbook::fill_areas()
   {
   const char * const msg = "Filling blocks...";
   bool first_post = true;
 
-  for( int index = 0; index < sblocks(); ++index )
+  for( long index = 0; index < sblocks(); ++index )
     {
     const Sblock & sb = sblock( index );
     if( !domain().includes( sb ) ) { if( domain() < sb ) break; else continue; }
@@ -54,14 +86,14 @@ int Fillbook::fill_areas( const std::string & filltypes )
       if( verbosity >= 0 )
         { show_status( b.pos(), msg, first_post ); first_post = false; }
       if( interrupted() ) return -1;
-      const int retval = fill_block( b );
+      const int retval = fill_block( Sblock( b, sb.status() ) );
       if( retval )					// write error
         {
-        if( !ignore_write_errors_ ) return retval;
+        if( !ignore_write_errors ) return retval;
         if( b.size() > hardbs() )	// fill the area a hardbs at a time
           { b.size( hardbs() ); continue; }
         }
-      if( !update_logfile( odes_ ) ) return -2;
+      if( !update_mapfile( odes_ ) ) return -2;
       b.pos( b.end() );
       if( b.end() > sb.end() ) b.crop( sb );
       }
@@ -74,17 +106,22 @@ int Fillbook::fill_areas( const std::string & filltypes )
 void Fillbook::show_status( const long long ipos, const char * const msg,
                             bool force )
   {
-  const char * const up = "\x1b[A";
+  const char * const up = "\x1B[A";
   if( t0 == 0 )
     {
     t0 = t1 = initial_time();
     first_size = last_size = filled_size;
     force = true;
-    std::printf( "\n\n\n" );
+    std::fputs( "\n\n\n", stdout );
     }
 
   if( ipos >= 0 ) last_ipos = ipos;
   const long t2 = std::time( 0 );
+  if( t2 < t1 )					// clock jumped back
+    {
+    t0 -= std::min( t0, t1 - t2 );
+    t1 = t2;
+    }
   if( t2 > t1 || force )
     {
     if( t2 > t1 )
@@ -95,13 +132,13 @@ void Fillbook::show_status( const long long ipos, const char * const msg,
       last_size = filled_size;
       }
     std::printf( "\r%s%s%s", up, up, up );
-    std::printf( "filled size: %10sB,  filled areas: %6u,  current rate: %9sB/s\n",
+    std::printf( "filled size: %10sB,  filled areas: %6ld,  current rate: %8sB/s\n",
                  format_num( filled_size ), filled_areas,
                  format_num( c_rate, 99999 ) );
-    std::printf( "remain size: %10sB,  remain areas: %6u,  average rate: %9sB/s\n",
+    std::printf( "remain size: %10sB,  remain areas: %6ld,  average rate: %8sB/s\n",
                  format_num( remaining_size ), remaining_areas,
                  format_num( a_rate, 99999 ) );
-    std::printf( "current pos: %10sB,  run time:  %9s\n",
+    std::printf( "current pos: %10sB,  run time: %11s\n",
                  format_num( last_ipos + offset() ), format_time( t1 - t0 ) );
     if( msg && msg[0] )
       {
@@ -111,17 +148,12 @@ void Fillbook::show_status( const long long ipos, const char * const msg,
       }
     std::fflush( stdout );
     }
-  else if( t2 < t1 )			// clock jumped back
-    {
-    t0 -= std::min( t0, t1 - t2 );
-    t1 = t2;
-    }
   }
 
 
 // Return values: 1 write error, 0 OK.
 //
-int Fillbook::do_fill( const int odes, const std::string & filltypes )
+int Fillbook::do_fill( const int odes )
   {
   filled_size = 0, remaining_size = 0;
   filled_areas = 0, remaining_areas = 0;
@@ -129,7 +161,7 @@ int Fillbook::do_fill( const int odes, const std::string & filltypes )
   if( current_status() != filling || !domain().includes( current_pos() ) )
     current_pos( 0 );
 
-  for( int i = 0; i < sblocks(); ++i )
+  for( long i = 0; i < sblocks(); ++i )
     {
     const Sblock & sb = sblock( i );
     if( !domain().includes( sb ) ) { if( domain() < sb ) break; else continue; }
@@ -145,36 +177,53 @@ int Fillbook::do_fill( const int odes, const std::string & filltypes )
   set_signals();
   if( verbosity >= 0 )
     {
-    std::printf( "Press Ctrl-C to interrupt\n" );
-    if( logfile_exists() )
+    std::fputs( "Press Ctrl-C to interrupt\n", stdout );
+    if( mapfile_exists() )
       {
-      std::printf( "Initial status (read from logfile)\n" );
-      std::printf( "filled size:    %10sB,  filled areas:    %7u\n",
+      std::fputs( "Initial status (read from mapfile)\n", stdout );
+      std::printf( "filled size:    %10sB,  filled areas:    %7ld\n",
                    format_num( filled_size ), filled_areas );
-      std::printf( "remaining size: %10sB,  remaining areas: %7u\n",
+      std::printf( "remaining size: %10sB,  remaining areas: %7ld\n",
                    format_num( remaining_size ), remaining_areas );
-      std::printf( "Current status\n" );
+      std::fputs( "Current status\n", stdout );
       }
     }
-  int retval = fill_areas( filltypes );
+  int retval = fill_areas();
   const bool signaled = ( retval == -1 );
   if( signaled ) retval = 0;
   if( verbosity >= 0 )
     {
     show_status( -1, ( retval || signaled ) ? 0 : "Finished", true );
-    if( retval == -2 ) std::printf( "\nLogfile error" );
-    else if( signaled ) std::printf( "\nInterrupted by user" );
+    if( retval == -2 ) std::fputs( "\nMapfile error", stdout );
+    else if( signaled ) std::fputs( "\nInterrupted by user", stdout );
     std::fputc( '\n', stdout );
+    std::fflush( stdout );
     }
-  if( retval == -2 ) retval = 1;		// logfile error
+  if( retval == -2 ) retval = 1;		// mapfile error
   else
     {
     if( retval == 0 && !signaled ) current_status( finished );
     compact_sblock_vector();
-    if( !update_logfile( odes_, true ) && retval == 0 ) retval = 1;
+    if( !update_mapfile( odes_, true ) && retval == 0 ) retval = 1;
     }
-  if( final_msg() ) show_error( final_msg(), final_errno() );
+  if( close( odes_ ) != 0 )
+    { show_error( "Can't close outfile", errno );
+      if( retval == 0 ) retval = 1; }
+  if( final_msg().size() ) show_error( final_msg().c_str(), final_errno() );
   if( retval ) return retval;		// errors have priority over signals
   if( signaled ) return signaled_exit();
   return 0;
   }
+
+
+bool Fillbook::read_buffer( const int ides )
+  {
+  const int rd = readblock( ides, iobuf(), softbs(), 0 );
+  if( rd <= 0 ) return false;
+  for( int i = rd; i < softbs(); i *= 2 )
+    {
+    const int size = std::min( i, softbs() - i );
+    std::memcpy( iobuf() + i, iobuf(), size );
+    }
+  return true;
+  }
